import { redirect } from "next/navigation";
import type { ExtendedAccessToken } from "@/types";
import { Search } from "lucide-react";

import { prisma } from "@/lib/prisma";
import { getCurrentUser } from "@/lib/session";
import { Badge } from "@/components/ui/badge";
import {
  Card,
  CardContent,
  CardDescription,
  CardHeader,
  CardTitle,
} from "@/components/ui/card";
import { Input } from "@/components/ui/input";
import {
  Table,
  TableBody,
  TableCell,
  TableHead,
  TableHeader,
  TableRow,
} from "@/components/ui/table";

async function getLogs(search?: string) {
  const thirtyDaysAgo = new Date();
  thirtyDaysAgo.setDate(thirtyDaysAgo.getDate() - 30);

  const logs = await prisma.accessToken.findMany({
    where: {
      expiresAt: {
        gte: thirtyDaysAgo,
      },
      ...(search
        ? {
            OR: [{ clientId: search }, { userId: search }],
          }
        : {}),
    },
    include: {
      client: {
        select: {
          name: true,
        },
      },
      user: {
        select: {
          username: true,
        },
      },
    },
    orderBy: [
      {
        expiresAt: "desc",
      },
    ],
    take: 100,
  });

  return logs as ExtendedAccessToken[];
}

export default async function LogsPage({
  searchParams,
}: {
  searchParams: { search?: string };
}) {
  const user = await getCurrentUser();
  if (!user || user.role !== "ADMIN") {
    redirect("/dashboard");
  }

  const logs = await getLogs(searchParams.search);

  return (
    <div className="mx-auto max-w-7xl px-4 py-8 sm:px-6 lg:px-8">
      <Card>
        <CardHeader>
          <div className="flex items-center justify-between">
            <div>
              <CardTitle>系统日志</CardTitle>
              <CardDescription>
                显示最近30天的授权记录和错误日志
              </CardDescription>
            </div>
            <div className="relative w-64">
              <Search className="absolute left-2 top-2.5 h-4 w-4 text-muted-foreground" />
              <form>
                <Input
                  placeholder="搜索日志..."
                  name="search"
                  defaultValue={searchParams.search}
                  className="pl-8"
                />
              </form>
            </div>
          </div>
        </CardHeader>
        <CardContent>
          <Table>
            <TableHeader>
              <TableRow>
                <TableHead>时间</TableHead>
                <TableHead>应用</TableHead>
                <TableHead>用户</TableHead>
                <TableHead>令牌</TableHead>
                <TableHead>状态</TableHead>
                <TableHead>错误信息</TableHead>
              </TableRow>
            </TableHeader>
            <TableBody>
              {logs.map((log: ExtendedAccessToken) => (
                <TableRow key={log.id}>
                  <TableCell>
                    {new Date(log.createdAt).toLocaleString()}
                  </TableCell>
                  <TableCell>{log.client?.name || "-"}</TableCell>
                  <TableCell>{log.user?.username || "-"}</TableCell>
                  <TableCell className="font-mono">
                    {log.id.slice(0, 8)}...
                  </TableCell>
                  <TableCell>
                    <Badge
                      variant={
                        log.error
                          ? "destructive"
                          : new Date(log.expiresAt) > new Date()
                            ? "default"
                            : "secondary"
                      }
                    >
                      {log.error
                        ? "错误"
                        : new Date(log.expiresAt) > new Date()
                          ? "有效"
                          : "过期"}
                    </Badge>
                  </TableCell>
                  <TableCell className="max-w-[200px] truncate text-red-500">
                    {log.error || "-"}
                  </TableCell>
                </TableRow>
              ))}
            </TableBody>
          </Table>
        </CardContent>
      </Card>
    </div>
  );
}
