"use client";

import * as React from "react";
import { useRouter, useSearchParams } from "next/navigation";
import { Loader2, MessageCircleCode } from "lucide-react";
import { signIn } from "next-auth/react";

import { cn } from "@/lib/utils";
import { useToast } from "@/hooks/use-toast";
import { buttonVariants } from "@/components/ui/button";

interface DiscourseData {
  sso_url: string;
}

export function UserAuthForm({
  className,
  ...props
}: React.HTMLAttributes<HTMLDivElement>) {
  const [isLoading, setIsLoading] = React.useState<boolean>(false);
  const router = useRouter();
  const { toast } = useToast();
  const searchParams = useSearchParams();

  async function handleSignIn() {
    try {
      // 如果有 SSO 参数，使用 credentials provider 登录
      if (searchParams.has("sso") && searchParams.has("sig")) {
        console.log("检测到 SSO 参数，使用 credentials provider 登录");
        const result = await signIn("credentials", {
          sso: searchParams.get("sso"),
          sig: searchParams.get("sig"),
          redirect: false,
        });

        if (result?.error) {
          throw new Error(result.error);
        }

        // 登录成功后刷新页面
        router.refresh();
        return;
      }

      // 如果有 OAuth 参数，发起 SSO 登录
      const body: Record<string, any> = {};
      if (searchParams.has("response_type") && searchParams.has("client_id")) {
        body.return_url = `/oauth/authorize?${searchParams.toString()}`;
        console.log("正在处理 OAuth 登录，return_url:", body.return_url);
      }

      console.log("发送登录请求...");
      const response = await fetch("/api/auth/q58", {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify(body),
      });

      if (!response.ok) {
        const errorData = await response.text();
        console.error("登录请求失败:", errorData);
        throw new Error(`登录请求失败: ${errorData}`);
      }

      const data = await response.json();
      console.log("获取到 SSO URL:", data.sso_url);
      window.location.href = data.sso_url;
    } catch (error) {
      console.error("登录错误:", error);
      toast({
        title: "登录失败",
        description:
          error instanceof Error
            ? error.message
            : "登录过程中发生错误，请稍后重试",
        variant: "destructive",
      });
      setIsLoading(false);
    }
  }

  return (
    <div className={cn("grid gap-3", className)} {...props}>
      <button
        type="button"
        className={cn(buttonVariants({ variant: "outline" }))}
        onClick={() => {
          setIsLoading(true);
          handleSignIn();
        }}
        disabled={isLoading}
      >
        {isLoading ? (
          <Loader2 className="mr-2 size-4 animate-spin" />
        ) : (
          <MessageCircleCode className="mr-2 size-4" />
        )}{" "}
        Q58论坛
      </button>
    </div>
  );
}
