import { Authorization } from "@prisma/client";

import { prisma } from "../prisma";

export async function createAuthorization(
  data: Omit<Authorization, "id" | "createdAt" | "updatedAt">,
) {
  await prisma.authorization.create({
    data,
  });
}

export async function findAuthorization(userId: string, clientId: string) {
  return await prisma.authorization.findUnique({
    where: {
      userId_clientId: {
        userId,
        clientId,
      },
    },
  });
}

export async function getAuthorizationsByClientId(clientId: string) {
  return prisma.authorization.findMany({
    where: {
      clientId,
    },
    include: {
      user: {
        select: {
          username: true,
          email: true,
          name: true,
          avatarUrl: true,
        },
      },
    },
    orderBy: {
      createdAt: "desc",
    },
  });
}

export async function updateAuthorizationStatus(
  userId: string,
  clientId: string,
  enabled: boolean,
) {
  return prisma.authorization.update({
    where: {
      userId_clientId: {
        userId,
        clientId,
      },
    },
    data: {
      enabled,
      updatedAt: new Date(),
    },
  });
}

export async function getAuthorizationStats(clientId: string) {
  const now = new Date();
  const thirtyDaysAgo = new Date(now.getTime() - 30 * 24 * 60 * 60 * 1000);

  const [total, activeLastMonth, newLastMonth] = await Promise.all([
    // 总授权用户数
    prisma.authorization.count({
      where: {
        clientId,
        enabled: true,
      },
    }),
    // 最近30天活跃用户数
    prisma.authorization.count({
      where: {
        clientId,
        enabled: true,
        lastUsedAt: {
          gte: thirtyDaysAgo,
        },
      },
    }),
    // 最近30天新增用户数
    prisma.authorization.count({
      where: {
        clientId,
        enabled: true,
        createdAt: {
          gte: thirtyDaysAgo,
        },
      },
    }),
  ]);

  return {
    total,
    activeLastMonth,
    newLastMonth,
  };
}

export async function getClientAuthorizationStats(clientId: string) {
  const stats = await getAuthorizationStats(clientId);

  // 获取最近的授权记录
  const recentAuthorizations = await prisma.authorization.findMany({
    where: {
      clientId,
      enabled: true,
    },
    orderBy: {
      lastUsedAt: "desc",
    },
    take: 5,
    include: {
      user: {
        select: {
          username: true,
          name: true,
        },
      },
    },
  });

  return {
    ...stats,
    recentAuthorizations,
  };
}
